window.Gleam.initSearch([{"doc":"clad","title":"clad","content":"# clad\n\n\n[![Package Version](https://img.shields.io/hexpm/v/clad)](https://hex.pm/packages/clad)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/clad/)\n\n\nCommand line argument decoders for Gleam.\n\n- Clad provides primitives to build a `dynamic.Decoder` for command line arguments.\n- Arguments can be specified with long names (`--name`) or short names (`-n`).\n- Values are decoded in the form `--name value` or `--name=value`.\n- Boolean flags do not an explicit value. If the flag exists it is `True`, and if it is missing it is `False`. (i.e. `--verbose`)\n\n\n## Usage\n\n```sh\ngleam add clad\n```\n\nThis program is in the [examples directory](https://github.com/ryanmiville/clad/tree/main/test/examples)\n\n```gleam\nimport argv\nimport clad\nimport gleam/bool\nimport gleam/dynamic\nimport gleam/int\nimport gleam/io\nimport gleam/string\n\ntype Order {\n  Order(flavors: List(String), scoops: Int, cone: Bool)\n}\n\nfn order_decoder() {\n  use flavors &lt;- clad.arg(&quot;flavor&quot;, &quot;f&quot;, dynamic.list(dynamic.string))\n  use scoops &lt;- clad.arg_with_default(&quot;scoops&quot;, &quot;s&quot;, dynamic.int, default: 1)\n  use cone &lt;- clad.toggle(&quot;cone&quot;, &quot;c&quot;)\n  clad.decoded(Order(flavors:, scoops:, cone:))\n}\n\npub fn main() {\n  let order =\n    order_decoder()\n    |&gt; clad.decode(argv.load().arguments)\n\n  case order {\n    Ok(order) -&gt; take_order(order)\n    _ -&gt;\n      io.println(\n        &quot;\nOptions:\n  -f, --flavor &lt;FLAVOR&gt;  Flavors of ice cream\n  -s, --scoops &lt;SCOOPS&gt;  Number of scoops per flavor [default: 1]\n  -c, --cone             Put ice cream in a cone\n      &quot;,\n      )\n  }\n}\n\nfn take_order(order: Order) {\n  let scoops = bool.guard(order.scoops == 1, &quot; scoop&quot;, fn() { &quot; scoops&quot; })\n  let container = bool.guard(order.cone, &quot;cone&quot;, fn() { &quot;cup&quot; })\n  let flavs = string.join(order.flavors, &quot; and &quot;)\n  io.println(\n    int.to_string(order.scoops)\n    &lt;&gt; scoops\n    &lt;&gt; &quot; of &quot;\n    &lt;&gt; flavs\n    &lt;&gt; &quot; in a &quot;\n    &lt;&gt; container\n    &lt;&gt; &quot;, coming right up!&quot;,\n  )\n}\n```\n\nRun the program\n\n```sh\n❯ gleam run -m examples/ice_cream -- -f vanilla\n1 scoop of vanilla in a cup, coming right up!\n❯ gleam run -m examples/ice_cream -- --flavor vanilla --flavor chocolate\n1 scoop of vanilla and chocolate in a cup, coming right up!\n❯ gleam run -m examples/ice_cream -- --flavor vanilla --flavor chocolate --scoops 2 --cone\n2 scoops of vanilla and chocolate in a cone, coming right up!\n❯ gleam run -m examples/ice_cream --\n\nOptions:\n  -f, --flavor &lt;FLAVOR&gt;  Flavors of ice cream\n  -s, --scoops &lt;SCOOPS&gt;  Number of scoops per flavor [default: 1]\n  -c, --cone             Put ice cream in a cone\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/clad&gt;.\n","url":"index.html"},{"doc":"clad","title":"arg","content":"pub fn arg(\n  long_name long_name: String,\n  short_name short_name: String,\n  of decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n  then next: fn(a) -&gt; fn(Dynamic) -&gt; Result(b, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(b, List(DecodeError))\n Decode an argument by either its long name (`--name`) or short name (`-n`).\n\n List arguments are represented by repeated values.\n\n # Examples\n \n // data: [&quot;--name&quot;, &quot;Lucy&quot;]\n use name &lt;- clad.arg(long_name: &quot;name&quot;, short_name: &quot;n&quot;, of: dynamic.string)\n // -&gt; &quot;Lucy&quot;\n \n \n // data: [&quot;-n&quot;, &quot;Lucy&quot;]\n use name &lt;- clad.arg(long_name: &quot;name&quot;, short_name: &quot;n&quot;, of: dynamic.string)\n // -&gt; &quot;Lucy&quot;\n \n \n // data: [&quot;-n&quot;, &quot;Lucy&quot;, &quot;-n&quot;, &quot;Joe&quot;]\n use name &lt;- clad.arg(\n   long_name: &quot;name&quot;,\n   short_name: &quot;n&quot;,\n   of: dynamic.list(dynamic.string)\n )\n // -&gt; [&quot;Lucy&quot;, &quot;Joe&quot;]\n \nSynonyms:\nclad.arg\nclad arg","url":"clad.html#arg"},{"doc":"clad","title":"arg_with_default","content":"pub fn arg_with_default(\n  long_name long_name: String,\n  short_name short_name: String,\n  of decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n  default default: a,\n  then next: fn(a) -&gt; fn(Dynamic) -&gt; Result(b, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(b, List(DecodeError))\n Decode an argument, returning a default value if the argument does not exist\n\n # Examples\n \n // data: [&quot;--name&quot;, &quot;Lucy&quot;]\n use name &lt;- clad.arg(\n   long_name: &quot;name&quot;,\n   short_name: &quot;n&quot;,\n   of: dynamic.string,\n   default: &quot;Joe&quot;\n )\n // -&gt; &quot;Lucy&quot;\n \n \n // data: []\n use name &lt;- clad.arg(\n   long_name: &quot;name&quot;,\n   short_name: &quot;n&quot;,\n   of: dynamic.string,\n   default: &quot;Joe&quot;\n )\n // -&gt; &quot;Joe&quot;\n \nSynonyms:\nclad.arg_with_default\nclad arg_with_default","url":"clad.html#arg_with_default"},{"doc":"clad","title":"decode","content":"pub fn decode(\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n  arguments: List(String),\n) -&gt; Result(a, List(DecodeError))\n Run a decoder on a list of command line arguments, decoding the value if it\n is of the desired type, or returning errors.\n\n This function pairs well with the [argv package](https://github.com/lpil/argv).\n\n # Examples\n \n {\n   use name &lt;- clad.arg(&quot;name&quot;, &quot;n&quot;, dynamic.string)\n   use email &lt;- clad.arg(&quot;email&quot;, &quot;e&quot;, dynamic.string),\n   clad.decoded(SignUp(name:, email:))\n }\n |&gt; clad.decode([&quot;-n&quot;, &quot;Lucy&quot;, &quot;--email=lucy@example.com&quot;])\n // -&gt; Ok(SignUp(name: &quot;Lucy&quot;, email: &quot;lucy@example.com&quot;))\n \n with argv:\n \n {\n   use name &lt;- clad.arg(&quot;name&quot;, &quot;n&quot;, dynamic.string)\n   use email &lt;- clad.arg(&quot;email&quot;, &quot;e&quot;, dynamic.string),\n   clad.decoded(SignUp(name:, email:))\n }\n |&gt; clad.decode(argv.load().arguments)\n \nSynonyms:\nclad.decode\nclad decode","url":"clad.html#decode"},{"doc":"clad","title":"decoded","content":"pub fn decoded(\n  value: a,\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n Creates a decoder which directly returns the provided value.\n Used to collect decoded values into a record.\n # Examples\n \n pub fn user_decoder() {\n   use name &lt;- clad.string(&quot;name&quot;, &quot;n&quot;)\n   clad.decoded(User(name:))\n }\n \nSynonyms:\nclad.decoded\nclad decoded","url":"clad.html#decoded"},{"doc":"clad","title":"long_name","content":"pub fn long_name(\n  long_name: String,\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n  next: fn(a) -&gt; fn(Dynamic) -&gt; Result(b, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(b, List(DecodeError))\n Decode an argument only by a long name\n\n # Examples\n \n // data: [&quot;--name&quot;, &quot;Lucy&quot;]\n use name &lt;- clad.long_name(&quot;name&quot;, dynamic.string)\n // -&gt; &quot;Lucy&quot;\n \nSynonyms:\nclad.long_name\nclad long_name","url":"clad.html#long_name"},{"doc":"clad","title":"short_name","content":"pub fn short_name(\n  short_name: String,\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n  next: fn(a) -&gt; fn(Dynamic) -&gt; Result(b, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(b, List(DecodeError))\n Decode an argument only by a short name\n\n # Examples\n \n // data: [&quot;-n&quot;, &quot;Lucy&quot;]\n use name &lt;- clad.short_name(&quot;n&quot;, dynamic.string)\n // -&gt; &quot;Lucy&quot;\n \nSynonyms:\nclad.short_name\nclad short_name","url":"clad.html#short_name"},{"doc":"clad","title":"toggle","content":"pub fn toggle(\n  long_name long_name: String,\n  short_name short_name: String,\n  then next: fn(Bool) -&gt;\n    fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n A decoder that decodes Bool arguments.\n\n Toggles do not need an explicit value. If the flag exists it is `True`,\n and `False` if it is missing. (i.e. `--verbose`)\n\n # Examples\n \n // data: [&quot;-v&quot;]\n use verbose &lt;- clad.toggle(long_name: &quot;verbose&quot;, short_name: &quot;v&quot;)\n // -&gt; True\n \n \n // data: []\n use verbose &lt;- clad.toggle(long_name: &quot;verbose&quot;, short_name: &quot;v&quot;)\n // -&gt; False\n \nSynonyms:\nclad.toggle\nclad toggle","url":"clad.html#toggle"},{"doc":"clad","title":"clad","content":" This module encodes a list of command line arguments as a `dynamic.Dynamic` and\n provides primitives to build a `dynamic.Decoder` to decode records from command line\n arguments.\n\n Arguments are parsed from long names (`--name`) or short names (`-n`).\n Values are decoded in the form `--name value` or `--name=value`.\n Boolean flags do not need an explicit value. If the flag exists it is `True`,\n and `False` if it is missing. (i.e. `--verbose`)\n\n # Examples\n\n ## Encoding\n\n Clad encodes the arguments without any knowledge of your target record.\n It cannot know if a field is intended to be a single, basic type or a\n list with a single item. Therefore it encodes everything as a list.\n\n All of the following get encoded the same:\n\n ```sh\n --name Lucy --count 3 --verbose\n --name Lucy --count 3 --verbose true\n --name=Lucy --count=3 --verbose=true\n ```\n\n ```gleam\n // {&quot;--name&quot;: [&quot;Lucy&quot;], &quot;--count&quot;: [3], &quot;--verbose&quot;: [true]}\n ```\n\n Since the target record is unknown, missing Bool arguments are not encoded at all:\n\n ```sh\n --name Lucy --count 3\n ```\n ```gleam\n // {&quot;--name&quot;: [&quot;Lucy&quot;], &quot;--count&quot;: [3]}\n ```\n\n There is no way to know that a long name and a short name are the same argument when encoding.\n So they are encoded as separate fields:\n\n ```sh\n --name Lucy -n Joe\n ```\n ```gleam\n // {&quot;--name&quot;: [&quot;Lucy&quot;], &quot;-n&quot;: [&quot;Joe&quot;]}\n ```\n\n ## Decoding Fields\n\n Clad provides the `arg` function to handle these quirks of the Dynamic representation.\n\n ```sh\n --name Lucy\n ```\n ```gleam\n use name &lt;- clad.arg(long_name: &quot;name&quot;, short_name: &quot;n&quot;, of: dynamic.string)\n // -&gt; &quot;Lucy&quot;\n ```\n ```sh\n -n Lucy\n ```\n ```gleam\n use name &lt;- clad.arg(long_name: &quot;name&quot;, short_name: &quot;n&quot;, of: dynamic.string)\n // -&gt; &quot;Lucy&quot;\n ```\n ```sh\n -n Lucy -n Joe\n ```\n ```gleam\n use names &lt;- clad.arg(&quot;name&quot;, &quot;n&quot;, of: dynamic.list(dynamic.string))\n // -&gt; [&quot;Lucy&quot;, &quot;Joe&quot;]\n ```\n\n Clad&#39;s `toggle` decoder only requires the name. Missing arguments are `False`:\n\n ```sh\n --verbose\n ```\n ```gleam\n use verbose &lt;- clad.toggle(long_name: &quot;verbose&quot;, short_name: &quot;v&quot;)\n // -&gt; True\n ```\n ```sh\n --name Lucy\n ```\n ```gleam\n use verbose &lt;- clad.toggle(long_name: &quot;verbose&quot;, short_name: &quot;v&quot;)\n // -&gt; False\n ```\n\n It&#39;s common for CLI&#39;s to have default values for arguments.\n This can be accomplished with a `dynamic.optional`, but\n the `arg_with_default` function is provided for convenience:\n\n ```sh\n --name Lucy\n ```\n ```gleam\n use count &lt;- clad.arg_with_default(\n   long_name: &quot;count&quot;,\n   short_name: &quot;c&quot;,\n   of: dynamic.int,\n   default: 1,\n )\n // -&gt; 1\n ```\n ## Decoding Records\n Clad&#39;s API is heavily inspired by (read: copied from) [toy](https://github.com/Hackder/toy).\n ```gleam\n fn arg_decoder() {\n   use name &lt;- clad.arg(&quot;name&quot;, &quot;n&quot;, dynamic.string)\n   use count &lt;- clad.arg_with_default(&quot;count&quot;, &quot;c&quot;, dynamic.int, 1)\n   use verbose &lt;- clad.toggle(&quot;verbose&quot;, &quot;v&quot;)\n   clad.decoded(Args(name:, count:, verbose:))\n }\n ```\n\n And then use it to decode the arguments:\n ```gleam\n // arguments: [&quot;--name&quot;, &quot;Lucy&quot;, &quot;--count&quot;, &quot;3&quot;, &quot;--verbose&quot;]\n\n let args =\n   arg_decoder()\n   |&gt; clad.decode(arguments)\n let assert Ok(Args(&quot;Lucy&quot;, 3, True)) = args\n ```\n\n Here are a few examples of arguments that would decode the same:\n\n ```sh\n --name Lucy --count 3 --verbose\n --name=Lucy -c 3 -v=true\n -n=Lucy -c=3 -v\n ```\n # Errors\n\n Clad returns the first error it encounters. If multiple fields have errors, only the first one will be returned.\n\n ```gleam\n // arguments: [&quot;--count&quot;, &quot;three&quot;]\n\n let args =\n   arg_decoder()\n   |&gt; clad.decode(arguments)\n let assert Error([DecodeError(&quot;field&quot;, &quot;nothing&quot;, [&quot;--name&quot;])]) = args\n ```\n\n If a field has a default value, but the argument is supplied with the incorrect type, an error will be returned rather than falling back on the default value.\n\n ```gleam\n // arguments: [&quot;-n&quot;, &quot;Lucy&quot; &quot;-c&quot;, &quot;three&quot;]\n\n let args =\n   arg_decoder()\n   |&gt; clad.decode(arguments)\n let assert Error([DecodeError(&quot;Int&quot;, &quot;String&quot;, [&quot;-c&quot;])]) = args\n ```","url":"clad.html"}]);